@@ -43,6 +43,8 @@
   :link '(url-link :tag "Denote homepage" "https://protesilaos.com/emacs/denote")
   :link '(url-link :tag "Denote Journal homepage" "https://protesilaos.com/emacs/denote-journal"))
 
+;;;; User options
+
 (defcustom denote-journal-directory
   (expand-file-name "journal" denote-directory)
   "Directory for storing daily journal entries.
@@ -64,6 +66,30 @@ It is used by `denote-journal-new-entry' (or related)."
   :type '(choice (string :tag "Keyword")
                  (repeat :tag "List of keywords" string)))
 
+(defcustom denote-journal-signature nil
+  "The signature to use in new journal entries.
+This is used by `denote-journal-new-entry' and related commands.
+
+The value can be one of the following:
+
+- nil, which means to not use a predefined signature;
+- a string, which is used as-is;
+- a function that returns a string, which is then used as-is.
+
+In the case of a function, users may wish to integrate the
+`denote-journal' package with `denote-sequence'.  For example, each new
+journal entry should be defined as a new parent sequence.  Thus:
+
+    (setq denote-journal-signature
+          (lambda ()
+            (denote-sequence-get-new 'parent)))"
+  :type '(choice
+          (const :tag "No predefined signature" nil)
+          (string :tag "The predefined signature to use for new entries")
+          (function :tag "Function that returns a string"))
+  :group 'denote-journal
+  :package-version '(denote . "0.2.0"))
+
 (defcustom denote-journal-title-format 'day-date-month-year-24h
   "Date format to construct the title with `denote-journal-new-entry'.
 The value it can take is either nil, a
@@ -107,6 +133,18 @@ custom string, or a symbol:
                  day-date-month-year-12h)
           (string :tag "Custom string with `format-time-string' specifiers")))
 
+(defcustom denote-journal-interval 'daily
+  "The interval used by `denote-journal-new-or-existing-entry'.
+The value is a symbol of `daily', `weekly', `monthly', or `yearly'.  Any
+other value is understood as `daily'."
+  :type '(choice
+          (const daily)
+          (const weekly)
+          (const monthly)
+          (const yearly))
+  :package-version '(denote . "0.2.0")
+  :group 'denote-journal)
+
 (defcustom denote-journal-hook nil
   "Normal hook called after `denote-journal-new-entry'.
 Use this to, for example, set a timer after starting a new
@@ -114,6 +152,8 @@ journal entry (refer to the `tmr' package on GNU ELPA)."
   :group 'denote-journal
   :type 'hook)
 
+;;;; Common helper functions
+
 (defun denote-journal-directory ()
   "Return the variable `denote-journal-directory' as a directory.
 If the path does not exist, then make it first."
@@ -131,6 +171,17 @@ If the path does not exist, then make it first."
       (list denote-journal-keyword)
     denote-journal-keyword))
 
+(defun denote-journal-signature ()
+  "Return the value of the variable `denote-journal-signature'."
+  (cond
+   ((stringp denote-journal-signature) denote-journal-signature)
+   ((functionp denote-journal-signature)
+    (if-let* ((value (funcall denote-journal-signature))
+              (_ (stringp value)))
+        value
+      (error "The `denote-journal-signature' is bound to a function that does not return a string")))
+   (t nil)))
+
 (defun denote-journal--keyword-regex ()
   "Return a regular expression string that matches the journal keyword(s)."
   (let ((keywords-sorted (mapcar #'regexp-quote (denote-keywords-sort (denote-journal-keyword)))))
@@ -177,6 +228,8 @@ is non-nil, prompt the user for a template among
     (or (alist-get 'journal denote-templates)
         (denote-template-prompt))))
 
+;;;; New entry without special conditions
+
 ;;;###autoload
 (defun denote-journal-new-entry (&optional date)
   "Create a new journal entry in variable `denote-journal-directory'.
@@ -198,13 +251,31 @@ is internally processed by `denote-valid-date-p'."
      (denote-journal-daily--title-format internal-date)
      (denote-journal-keyword)
      nil nil date
-     (denote-journal--get-template))
+     (denote-journal--get-template)
+     (denote-journal-signature))
     (run-hooks 'denote-journal-hook)))
 
-(defun denote-journal--filename-date-regexp (&optional date)
-  "Regular expression to match journal entries for today or optional DATE.
+;;;; New or existing entry based on `denote-journal-interval'
+
+(defun denote-journal--filename-regexp (date interval)
+  "Regular expression for Denote files with DATE matched to INTERVAL.
+INTERVAL is one among the symbols used by `denote-journal-interval'.
 DATE has the same format as that returned by `denote-valid-date-p'."
-  (let* ((identifier (format "%sT[0-9]\\{6\\}" (format-time-string "%Y%m%d" date)))
+  (let* ((identifier
+          (pcase interval
+            ('weekly
+             (let* ((day-of-week (string-to-number (format-time-string "%u" date)))
+                    (monday (time-subtract date (seconds-to-time (* (1- day-of-week) 24 3600))))
+                    (date-strings
+                     (let ((list nil))
+                       (dotimes (i 7)
+                         (push (format-time-string "%Y%m%d" (time-add monday (seconds-to-time (* i 24 3600)))) list))
+                       (nreverse list)))
+                    (date-regexp (regexp-opt date-strings)))
+               (format "%sT[0-9]\\{6\\}" date-regexp)))
+            ('monthly (format "%s[0-9]\\{2\\}T[0-9]\\{6\\}" (format-time-string "%Y%m" date)))
+            ('yearly (format "%s[0-9]\\{4\\}T[0-9]\\{6\\}" (format-time-string "%Y" date)))
+            (_ (format "%sT[0-9]\\{6\\}" (format-time-string "%Y%m%d" date)))))
          (order denote-file-name-components-order)
          (id-index (seq-position order 'identifier))
          (kw-index (seq-position order 'keywords)))
@@ -212,11 +283,31 @@ DATE has the same format as that returned by `denote-valid-date-p'."
         (format "%s.*?%s" identifier (denote-journal--keyword-regex))
       (format "%s.*?@@%s" (denote-journal--keyword-regex) identifier))))
 
-(defun denote-journal--entry-today (&optional date)
-  "Return list of files matching a journal for today or optional DATE.
+(defun denote-journal--date-in-interval-p (date interval)
+  "Return DATE if it is within the INTERVAL else nil.
+INTERVAL is one among the symbols used by `denote-journal-interval'.
+DATE has the same format as that returned by `denote-valid-date-p'."
+  (if-let* ((date (denote-valid-date-p date))
+            (current (current-time))
+            (specifiers (pcase interval
+                          ('weekly "%Y-%V")
+                          ('monthly "%Y-%m")
+                          ('yearly "%Y")
+                          (_ t))))
+      (cond
+       ((eq specifiers t)
+        date)
+       ((string= (format-time-string specifiers date)
+                 (format-time-string specifiers current))
+        date))
+    (error "The date `%s' does not satisfy `denote-valid-date-p'" date)))
+
+(defun denote-journal--get-entry (date interval)
+  "Return list of files matching a journal for DATE given INTERVAL.
+INTERVAL is one among the symbols used by `denote-journal-interval'.
 DATE has the same format as that returned by `denote-valid-date-p'."
   (let ((denote-directory (denote-journal-directory)))
-    (denote-directory-files (denote-journal--filename-date-regexp date))))
+    (denote-directory-files (denote-journal--filename-regexp date interval))))
 
 (defun denote-journal-select-file-prompt (files)
   "Prompt for file among FILES if >1, else return the `car'.
@@ -230,7 +321,7 @@ Perform the operation relative to the variable `denote-journal-directory'."
     (concat denote-directory file)))
 
 ;;;###autoload
-(defun denote-journal-path-to-new-or-existing-entry (&optional date)
+(defun denote-journal-path-to-new-or-existing-entry (&optional date interval)
   "Return path to existing or new journal file.
 With optional DATE, do it for that date, else do it for today.  DATE is
 a string and has the same format as that covered in the documentation of
@@ -238,9 +329,14 @@ the `denote' function.  It is internally processed by `denote-valid-date-p'.
 
 If there are multiple journal entries for the date, prompt for one among
 them using minibuffer completion.  If there is only one, return it.  If
-there is no journal entry, create it."
-  (let* ((internal-date (or (denote-valid-date-p date) (current-time)))
-         (files (denote-journal--entry-today internal-date))
+there is no journal entry, create it.
+
+With optional INTERVAL as a symbol among those accepted by
+`denote-journal-interval', match DATE to INTERVAL and then return the
+results accordingly.  If INTERVAL is nil, then it has the same measing
+as `daily', per `denote-journal-interval'."
+  (let* ((internal-date (denote-journal--date-in-interval-p (or date (current-time)) interval))
+         (files (denote-journal--get-entry internal-date interval))
          (denote-kill-buffers nil))
     (if files
         (denote-journal-select-file-prompt files)
@@ -266,12 +362,20 @@ date selection module.
 
 When called from Lisp, DATE is a string and has the same format
 as that covered in the documentation of the `denote' function.
-It is internally processed by `denote-valid-date-p'."
+It is internally processed by `denote-valid-date-p'.
+
+Consult the user option `denote-journal-interval' to determine when to
+create a new file or visit an existing one."
   (interactive
    (list
     (when current-prefix-arg
       (denote-date-prompt))))
-  (find-file (denote-journal-path-to-new-or-existing-entry date)))
+  (if-let* ((date-to-use (or date (current-time)))
+            (file (denote-journal-path-to-new-or-existing-entry date-to-use denote-journal-interval)))
+      (find-file file)
+    (error "Cannot get a new or existing journal entry")))
+
+;;;; Link or create functionality
 
 ;;;###autoload
 (defun denote-journal-link-or-create-entry (&optional date id-only)
@@ -366,7 +470,7 @@ CALENDAR-DATE is a list of three numbers, in the form of (MONTH DAY YEAR)."
   "Return path to Denote journal entry corresponding to CALENDAR-DATE.
 CALENDAR-DATE is a list of three numbers, in the form of (MONTH DAY YEAR)."
   (when-let* ((date (denote-journal-calendar--date-to-time calendar-date)))
-    (denote-journal--entry-today date)))
+    (denote-journal--get-entry date)))
 
 (defun denote-journal-calendar-find-file ()
   "Show the Denote journal entry for the `calendar' date at point.
